@@ -32,6 +32,9 @@ import random
 #from awlsim.core.instructions.all_insns cimport * #@cy
 
 from awlsim.common.cpuspecs import *
+
+from awlsim.library.libentry import *
+
 from awlsim.core.parser import *
 from awlsim.core.symbolparser import *
 from awlsim.core.datatypes import *
@@ -238,7 +241,6 @@ class S7CPU(object): #+cdef
 			# Add interface references to the parameter assignments.
 			for param in insn.params:
 				param.interface = calledCodeBlock.interface
-				param.instanceDB = calledDataBlock
 
 	def __finalizeCodeBlocks(self):
 		for block in self.__allUserCodeBlocks():
@@ -280,7 +282,7 @@ class S7CPU(object): #+cdef
 								   udtNumber)
 			if udtNumber in self.udts:
 				raise AwlSimError("Multiple definitions of "\
-					"UDT %d" % udtNumber)
+					"UDT %d." % udtNumber)
 			rawUDT.index = udtNumber
 			self.udts[udtNumber] = UDT.makeFromRaw(rawUDT)
 
@@ -295,7 +297,7 @@ class S7CPU(object): #+cdef
 			if obNumber in self.obs and\
 			   self.obs[obNumber].insns:
 				raise AwlSimError("Multiple definitions of "\
-					"OB %d" % obNumber)
+					"OB %d." % obNumber)
 			rawOB.index = obNumber
 			ob = translator.translateCodeBlock(rawOB, OB)
 			self.obs[obNumber] = ob
@@ -311,8 +313,14 @@ class S7CPU(object): #+cdef
 			fbNumber, sym = resolver.resolveBlockName((AwlDataType.TYPE_FB_X,),
 								  fbNumber)
 			if fbNumber in self.fbs:
+				extra = ""
+				if self.fbs[fbNumber].isLibraryBlock:
+					extra = "\nFB %d is already defined by an "\
+						"imported library block (%s)." % (
+						fbNumber,
+						self.fbs[fbNumber].libraryName)
 				raise AwlSimError("Multiple definitions of "\
-					"FB %d" % fbNumber)
+					"FB %d.%s" % (fbNumber, extra))
 			rawFB.index = fbNumber
 			fb = translator.translateCodeBlock(rawFB, FB)
 			self.fbs[fbNumber] = fb
@@ -322,8 +330,14 @@ class S7CPU(object): #+cdef
 			fcNumber, sym = resolver.resolveBlockName((AwlDataType.TYPE_FC_X,),
 								  fcNumber)
 			if fcNumber in self.fcs:
+				extra = ""
+				if self.fcs[fcNumber].isLibraryBlock:
+					extra = "\nFC %d is already defined by an "\
+						"imported library block (%s)." % (
+						fcNumber,
+						self.fcs[fcNumber].libraryName)
 				raise AwlSimError("Multiple definitions of "\
-					"FC %d" % fcNumber)
+					"FC %d.%s" % (fcNumber, extra))
 			rawFC.index = fcNumber
 			fc = translator.translateCodeBlock(rawFC, FC)
 			self.fcs[fcNumber] = fc
@@ -356,11 +370,42 @@ class S7CPU(object): #+cdef
 								  dbNumber)
 			if dbNumber in self.dbs:
 				raise AwlSimError("Multiple definitions of "\
-					"DB %d" % dbNumber)
+					"DB %d." % dbNumber)
 			rawDB.index = dbNumber
 			db = translator.translateDB(rawDB)
 			self.dbs[dbNumber] = db
 
+	def loadLibraryBlock(self, libSelection):
+		# Get the block class from the library.
+		libEntryCls = AwlLib.getEntryBySelection(libSelection)
+		assert(not libEntryCls.isSystemBlock)
+
+		# Get the effective block index.
+		effIndex = libSelection.getEffectiveEntryIndex()
+		if effIndex < 0:
+			effIndex = libSelection.getEntryIndex()
+
+		# Create and translate the block
+		translator = AwlTranslator(self)
+		if libEntryCls.isFC:
+			block = libEntryCls(index = effIndex)
+			if block.index in self.fcs:
+				raise AwlSimError("Error while loading library "
+					"block FC %d: Block FC %d is already "
+					"loaded." % (block.index, block.index))
+			block = translator.translateLibraryCodeBlock(block)
+			self.fcs[block.index] = block
+		elif libEntryCls.isFB:
+			block = libEntryCls(index = effIndex)
+			if block.index in self.fbs:
+				raise AwlSimError("Error while loading library "
+					"block FB %d: Block FB %d is already "
+					"loaded." % (block.index, block.index))
+			block = translator.translateLibraryCodeBlock(block)
+			self.fbs[block.index] = block
+		else:
+			assert(0)
+
 	def loadSymbolTable(self, symbolTable):
 		self.symbolTable.merge(symbolTable)
 
@@ -485,7 +530,7 @@ class S7CPU(object): #+cdef
 		self.ar1.reset()
 		self.ar2.reset()
 		self.statusWord.reset()
-		self.callStack = [ CallStackElem(self, block, None, (), True) ]
+		self.callStack = [ CallStackElem(self, block, None, None, (), True) ]
 		cse = self.callStackTop = self.callStack[-1]
 		if self.__obTempPresetsEnabled:
 			# Populate the TEMP region
@@ -691,45 +736,41 @@ class S7CPU(object): #+cdef
 
 	def __call_FC(self, blockOper, dbOper, parameters):
 		fc = self.fcs[blockOper.value.byteOffset]
-		return CallStackElem(self, fc, None, parameters)
+		return CallStackElem(self, fc, None, None, parameters)
 
 	def __call_RAW_FC(self, blockOper, dbOper, parameters):
 		fc = self.fcs[blockOper.value.byteOffset]
-		return CallStackElem(self, fc, None, (), True)
+		return CallStackElem(self, fc, None, None, (), True)
 
 	def __call_FB(self, blockOper, dbOper, parameters):
 		fb = self.fbs[blockOper.value.byteOffset]
 		db = self.dbs[dbOper.value.byteOffset]
-		cse = CallStackElem(self, fb, db, parameters)
+		cse = CallStackElem(self, fb, db, AwlOffset(), parameters)
 		self.dbRegister, self.diRegister = self.diRegister, db
-		# Set AR2 to multi-instance base DBX 0.0
-		self.ar2.set(AwlIndirectOp.AREA_DB)
 		return cse
 
 	def __call_RAW_FB(self, blockOper, dbOper, parameters):
 		fb = self.fbs[blockOper.value.byteOffset]
-		return CallStackElem(self, fb, self.diRegister, (), True)
+		return CallStackElem(self, fb, self.diRegister, None, (), True)
 
 	def __call_SFC(self, blockOper, dbOper, parameters):
 		sfc = self.sfcs[blockOper.value.byteOffset]
-		return CallStackElem(self, sfc, None, parameters)
+		return CallStackElem(self, sfc, None, None, parameters)
 
 	def __call_RAW_SFC(self, blockOper, dbOper, parameters):
 		sfc = self.sfcs[blockOper.value.byteOffset]
-		return CallStackElem(self, sfc, None, (), True)
+		return CallStackElem(self, sfc, None, None, (), True)
 
 	def __call_SFB(self, blockOper, dbOper, parameters):
 		sfb = self.sfbs[blockOper.value.byteOffset]
 		db = self.dbs[dbOper.value.byteOffset]
-		cse = CallStackElem(self, sfb, db, parameters)
+		cse = CallStackElem(self, sfb, db, AwlOffset(), parameters)
 		self.dbRegister, self.diRegister = self.diRegister, db
-		# Set AR2 to multi-instance base DBX 0.0
-		self.ar2.set(AwlIndirectOp.AREA_DB)
 		return cse
 
 	def __call_RAW_SFB(self, blockOper, dbOper, parameters):
 		sfb = self.sfbs[blockOper.value.byteOffset]
-		return CallStackElem(self, sfb, self.diRegister, (), True)
+		return CallStackElem(self, sfb, self.diRegister, None, (), True)
 
 	def __call_INDIRECT(self, blockOper, dbOper, parameters):
 		blockOper = blockOper.resolve()
@@ -741,12 +782,16 @@ class S7CPU(object): #+cdef
 					  blockOper.value.byteOffset)
 
 	def __call_MULTI_FB(self, blockOper, dbOper, parameters):
-		pass#TODO
-		raise AwlSimError("Multi-instance calls not supported, yet")
+		fb = self.fbs[blockOper.value.fbNumber]
+		cse = CallStackElem(self, fb, self.diRegister, blockOper.value, parameters)
+		self.dbRegister = self.diRegister
+		return cse
 
 	def __call_MULTI_SFB(self, blockOper, dbOper, parameters):
-		pass#TODO
-		raise AwlSimError("Multi-instance calls not supported, yet")
+		sfb = self.sfbs[blockOper.value.fbNumber]
+		cse = CallStackElem(self, sfb, self.diRegister, blockOper.value, parameters)
+		self.dbRegister = self.diRegister
+		return cse
 
 	__callHelpers = {
 		AwlOperator.BLKREF_FC	: __call_FC,
