@@ -1,5 +1,5 @@
 from __future__ import division, absolute_import, print_function, unicode_literals
-from awlsim.core.compat import *
+from awlsim.common.compat import *
 
-from awlsim.coreserver.client import *
+from awlsim.coreserver.messages import *
 from awlsim.coreserver.server import *
