@@ -76,6 +76,7 @@ class CpuWidget(QWidget):
 	STATE_INIT	= EnumGen.item
 	STATE_LOAD	= EnumGen.item
 	STATE_RUN	= EnumGen.item
+	STATE_EXCEPTION	= EnumGen.item
 	EnumGen.end
 
 	def __init__(self, mainWidget, parent=None):
@@ -224,12 +225,11 @@ class CpuWidget(QWidget):
 
 		self.__setState(self.STATE_INIT)
 		self.runButton.setChecked(True)
-		self.runButton.setEnabled(False) # Redraws the radio button
-		self.runButton.setEnabled(True)
 
 		project = self.mainWidget.getProject()
 		awlSources = self.mainWidget.projectWidget.getAwlSources()
 		symTabSources = self.mainWidget.projectWidget.getSymTabSources()
+		libSelections = self.mainWidget.projectWidget.getLibSelections()
 		if not all(awlSources) or not all(symTabSources):
 			self.stop()
 			return
@@ -278,15 +278,19 @@ class CpuWidget(QWidget):
 			client.loadHardwareModule("dummy")
 			for symTabSource in symTabSources:
 				client.loadSymbolTable(symTabSource)
+			for libSel in libSelections:
+				client.loadLibraryBlock(libSel)
 			for awlSource in awlSources:
 				client.loadCode(awlSource)
 			client.setRunState(True)
 		except AwlParserError as e:
+			self.__setState(self.STATE_EXCEPTION)
 			MessageBox.handleAwlParserError(self, e)
 			self.stop()
 			client.shutdown()
 			return
 		except AwlSimError as e:
+			self.__setState(self.STATE_EXCEPTION)
 			MessageBox.handleAwlSimError(self,
 				"Error while loading code", e)
 			self.stop()
@@ -308,6 +312,7 @@ class CpuWidget(QWidget):
 				# Process GUI events
 				QApplication.processEvents(QEventLoop.AllEvents)
 		except AwlSimError as e:
+			self.__setState(self.STATE_EXCEPTION)
 			MessageBox.handleAwlSimError(self,
 				"Error while executing code", e)
 			self.stop()
