@@ -25,6 +25,7 @@ from awlsim.common.compat import *
 from awlsim.gui.util import *
 from awlsim.gui.cpuwidget import *
 from awlsim.gui.sourcecodeedit import *
+from awlsim.gui.codevalidator import *
 
 
 class EditSubWidget(QWidget):
@@ -45,9 +46,7 @@ class EditSubWidget(QWidget):
 
 	def getPainter(self):
 		p = QPainter(self)
-		font = p.font()
-		setFixedFontParams(font)
-		p.setFont(font)
+		p.setFont(self.font())
 		return p
 
 	def contextMenuEvent(self, ev):
@@ -335,20 +334,27 @@ class CpuStatsContextMenu(QMenu):
 		self.closed.emit()
 
 class EditWidget(SourceCodeEdit):
+	# Signal: Emitted, if the source code changed.
 	codeChanged = Signal()
+	# Signal: Emitted, if the visible source lines changed (e.g. scrolled)
 	visibleRangeChanged = Signal()
+	# Signal: Keyboard focus in/out event.
+	focusChanged = Signal(bool)
 
 	__aniChars = ( ' ', '.', 'o', '0', 'O', '0', 'o', '.' )
 
 	def __init__(self, parent=None):
 		SourceCodeEdit.__init__(self, parent)
 
+		self.__validatorResults = []
+		self.__validatorTimer = QTimer(self)
+		self.__validatorTimer.setSingleShot(True)
+		self.__validatorTimer.timeout.connect(self.__checkValidator)
+
 		self.__aniTimer = QTimer(self)
 		self.__aniTimer.setSingleShot(False)
 		self.__aniTimer.timeout.connect(self.__animation)
 
-		self.__updateFonts()
-
 		self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
 		self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
 		self.setLineWrapMode(SourceCodeEdit.NoWrap)
@@ -358,6 +364,8 @@ class EditWidget(SourceCodeEdit):
 		self.lineNumWidget = LineNumSubWidget(self)
 		self.cpuStatsWidget = CpuStatsSubWidget(self)
 
+		self.__updateFont(getDefaultFixedFont())
+
 		self.__source = AwlSource(name = "Unnamed source")
 		self.__needSourceUpdate = True
 
@@ -389,6 +397,17 @@ class EditWidget(SourceCodeEdit):
 		self.cpuStatsWidget.contextMenuReq.connect(self.__cpuStatsContextMenuPopup)
 		self.__cpuStatsMenu.closed.connect(self.__cpuStatsContextMenuClosed)
 
+	def shutdown(self):
+		while self.__validatorResults:
+			self.__checkValidator(rearm=False)
+			QApplication.processEvents(QEventLoop.ExcludeUserInputEvents)
+		self.__validatorTimer.stop()
+		self.__validatorTimer = None
+
+		# Make sure all QTimer objects are collected.
+		# This avoids crashes within Qt.
+		garbageCollector.collect()
+
 	def setSource(self, source):
 		self.__textChangeBlocked += 1
 		try:
@@ -434,6 +453,17 @@ class EditWidget(SourceCodeEdit):
 			self.__updateSource()
 		return self.__source
 
+	def setSettings(self, guiSettings):
+		self.enableAutoIndent(guiSettings.getEditorAutoIndentEn())
+		self.enablePasteIndent(guiSettings.getEditorPasteIndentEn())
+		self.enableValidation(guiSettings.getEditorValidationEn())
+		font = getDefaultFixedFont()
+		fontString = guiSettings.getEditorFont()
+		if fontString:
+			font.fromString(fontString)
+			font.setStyleHint(QFont.Courier)
+		self.__updateFont(font)
+
 	def runStateChanged(self, newState):
 		self.__runStateCopy = newState
 		if newState == CpuWidget.STATE_INIT:
@@ -469,15 +499,30 @@ class EditWidget(SourceCodeEdit):
 				lastVisibleLine = lineNr
 		return firstVisibleLine, lastVisibleLine
 
-	def __updateFonts(self):
+	def __updateFont(self, font):
+		# Set the font for new text
 		fmt = self.currentCharFormat()
-		font = fmt.font()
-		setFixedFontParams(font)
 		fmt.setFont(font)
 		self.setCurrentCharFormat(fmt)
-		font = self.font()
-		setFixedFontParams(font)
+
+		# Reformat the existing text
+		cursor = self.textCursor()
+		cursor.select(QTextCursor.Document)
+		fmt = cursor.charFormat()
+		fmt.setFont(font)
+		cursor.setCharFormat(fmt)
+
+		# Set the font for other window elements
 		self.setFont(font)
+		self.update()
+		self.headerWidget.setFont(font)
+		self.headerWidget.update()
+		self.lineNumWidget.setFont(font)
+		self.lineNumWidget.update()
+		self.cpuStatsWidget.setFont(font)
+		self.cpuStatsWidget.update()
+
+		# Cache metrics
 		self.__charHeight = self.fontMetrics().height()
 		self.setTabStopWidth(self.fontMetrics().width("X") * 8)
 
@@ -642,6 +687,14 @@ class EditWidget(SourceCodeEdit):
 		SourceCodeEdit.resizeEvent(self, ev)
 		self.__updateGeo()
 
+	def focusInEvent(self, ev):
+		SourceCodeEdit.focusInEvent(self, ev)
+		self.focusChanged.emit(True)
+
+	def focusOutEvent(self, ev):
+		SourceCodeEdit.focusOutEvent(self, ev)
+		self.focusChanged.emit(False)
+
 	__runStateToText = {
 		CpuWidget.STATE_STOP		: "-- CPU STOPPED --",
 		CpuWidget.STATE_INIT		: "Initializing simulator...",
@@ -712,9 +765,26 @@ class EditWidget(SourceCodeEdit):
 					   statsEnt.getText(self.__cpuStatsMask))
 
 	def __textChanged(self):
-		self.__updateFonts()
 		if self.__textChangeBlocked:
 			return
 		self.__needSourceUpdate = True
 		self.codeChanged.emit()
 		self.resetCpuStats()
+
+	# Validation callback. Overridden subclass method.
+	def validateText(self, text, currentLineNr):
+		result = AwlValidator.get().enqueue(text)
+		self.__validatorResults.append(result)
+		if not self.__validatorTimer.isActive():
+			self.__validatorTimer.start(50)
+
+	def __checkValidator(self, rearm=True):
+		if not self.__validatorResults or\
+		   not self.__validatorResults[0].ready():
+			return
+		result = self.__validatorResults.pop(0)
+
+		self.setErraticLines(result.getErrLines())
+
+		if self.__validatorResults and rearm:
+			self.__validatorTimer.start(25)
