@@ -42,6 +42,13 @@ from awlsim.gui.icons.stdlib import *
 from awlsim.gui.icons.prefs import *
 from awlsim.gui.icons.network import *
 from awlsim.gui.icons.textsource import *
+from awlsim.gui.icons.exit import *
+from awlsim.gui.icons.undo import *
+from awlsim.gui.icons.redo import *
+from awlsim.gui.icons.browser import *
+from awlsim.gui.icons.cut import *
+from awlsim.gui.icons.copy import *
+from awlsim.gui.icons.paste import *
 
 import base64
 
@@ -65,12 +72,22 @@ __icons = {
 	"prefs"		: icon_prefs,
 	"network"	: icon_network,
 	"textsource"	: icon_textsource,
+	"exit"		: icon_exit,
+	"undo"		: icon_undo,
+	"redo"		: icon_redo,
+	"browser"	: icon_browser,
+	"cut"		: icon_cut,
+	"copy"		: icon_copy,
+	"paste"		: icon_paste,
 }
 
 def getIcon(iconName):
 	global __icons
 
-	icon = __icons[iconName]
+	try:
+		icon = __icons[iconName]
+	except KeyError:
+		return QIcon()
 	if isinstance(icon, QIcon):
 		# The icon is alredy cached. Return it.
 		return icon
