@@ -28,14 +28,27 @@ import os
 from awlsim.gui.util import *
 from awlsim.gui.editwidget import *
 from awlsim.gui.projectwidget import *
+from awlsim.gui.guiconfig import *
 from awlsim.gui.cpuconfig import *
 from awlsim.gui.coreconfig import *
 from awlsim.gui.icons import *
 
 
 class MainWidget(QWidget):
+	# Signal: Dirty-status changed
 	dirtyChanged = Signal(bool)
+	# Signal: CPU run state changed
 	runStateChanged = Signal(int)
+	# Signal: Source text focus changed
+	textFocusChanged = Signal(bool)
+	# Signal: Selected project resource changed
+	selResourceChanged = Signal(int)
+	# Signal: UndoAvailable state changed
+	undoAvailableChanged = Signal(bool)
+	# Signal: RedoAvailable state changed
+	redoAvailableChanged = Signal(bool)
+	# Signal: CopyAvailable state changed
+	copyAvailableChanged = Signal(bool)
 
 	def __init__(self, parent=None):
 		QWidget.__init__(self, parent)
@@ -61,6 +74,11 @@ class MainWidget(QWidget):
 		self.projectWidget.symTabChanged.connect(self.__somethingChanged)
 		self.projectWidget.libTableChanged.connect(self.__somethingChanged)
 		self.projectWidget.visibleLinesChanged.connect(self.cpuWidget.updateVisibleLineRange)
+		self.projectWidget.textFocusChanged.connect(self.textFocusChanged)
+		self.projectWidget.selResourceChanged.connect(self.selResourceChanged)
+		self.projectWidget.undoAvailableChanged.connect(self.undoAvailableChanged)
+		self.projectWidget.redoAvailableChanged.connect(self.redoAvailableChanged)
+		self.projectWidget.copyAvailableChanged.connect(self.copyAvailableChanged)
 		self.cpuWidget.runStateChanged.connect(self.__runStateChanged)
 		self.cpuWidget.onlineDiagChanged.connect(self.projectWidget.handleOnlineDiagChange)
 		self.cpuWidget.haveInsnDump.connect(self.projectWidget.handleInsnDump)
@@ -191,9 +209,18 @@ class MainWidget(QWidget):
 		else:
 			return self.saveFile(self.filename)
 
+	def guiConfig(self):
+		project = self.getProject()
+		dlg = GuiConfigDialog(self)
+		dlg.loadFromProject(project)
+		if dlg.exec_() == dlg.Accepted:
+			dlg.saveToProject(project)
+			self.projectWidget.setSettings(project.getGuiSettings())
+			self.__somethingChanged()
+
 	def cpuConfig(self):
 		project = self.getProject()
-		dlg = CpuConfigDialog(self, self.simClient)
+		dlg = CpuConfigDialog(self)
 		dlg.loadFromProject(project)
 		if dlg.exec_() == dlg.Accepted:
 			dlg.saveToProject(project)
@@ -229,6 +256,21 @@ class MainWidget(QWidget):
 	def openLibrary(self):
 		self.projectWidget.openLibrary()
 
+	def undo(self):
+		self.projectWidget.undo()
+
+	def redo(self):
+		self.projectWidget.redo()
+
+	def cut(self):
+		self.projectWidget.clipboardCut()
+
+	def copy(self):
+		self.projectWidget.clipboardCopy()
+
+	def paste(self):
+		self.projectWidget.clipboardPaste()
+
 class MainWindow(QMainWindow):
 	@classmethod
 	def start(cls,
@@ -253,7 +295,16 @@ class MainWindow(QMainWindow):
 		self.saveAct = menu.addAction(getIcon("save"), "&Save project", self.save)
 		menu.addAction(getIcon("save"), "&Save project as...", self.saveAs)
 		menu.addSeparator()
-		menu.addAction("&Exit...", self.close)
+		menu.addAction(getIcon("exit"), "&Exit...", self.close)
+		self.menuBar().addMenu(menu)
+
+		menu = QMenu("&Edit", self)
+		self.undoAct = menu.addAction(getIcon("undo"), "&Undo", self.undo)
+		self.redoAct = menu.addAction(getIcon("redo"), "&Redo", self.redo)
+		menu.addSeparator()
+		self.cutAct = menu.addAction(getIcon("cut"), "&Cut", self.cut)
+		self.copyAct = menu.addAction(getIcon("copy"), "&Copy", self.copy)
+		self.pasteAct = menu.addAction(getIcon("paste"), "&Paste", self.paste)
 		self.menuBar().addMenu(menu)
 
 		menu = QMenu("&Library", self)
@@ -267,15 +318,18 @@ class MainWindow(QMainWindow):
 		menu.addAction(getIcon("textsource"), "Insert FC C&ALL template...", self.insertFCcall)
 		menu.addAction(getIcon("textsource"), "Insert FB CA&LL template...", self.insertFBcall)
 		menu.addSeparator()
-		menu.addAction(getIcon("stdlib"), "&Standard library...", self.openLibrary)
+		self.libAct = menu.addAction(getIcon("stdlib"), "&Standard library...", self.openLibrary)
 		self.menuBar().addMenu(menu)
 
-		menu = QMenu("&PLC", self)
+		menu = QMenu("&Settings", self)
 		menu.addAction(getIcon("network"), "&Server connection...", self.coreConfig)
 		menu.addAction(getIcon("prefs"), "&CPU config...", self.cpuConfig)
+		menu.addAction(getIcon("prefs"), "&User interface...", self.guiConfig)
 		self.menuBar().addMenu(menu)
 
 		menu = QMenu("&Help", self)
+		menu.addAction(getIcon("browser"), "Project &homepage...", self.projectHome)
+		menu.addSeparator()
 		menu.addAction(getIcon("cpu"), "&About...", self.about)
 		self.menuBar().addMenu(menu)
 
@@ -284,13 +338,30 @@ class MainWindow(QMainWindow):
 		self.tb.addAction(getIcon("open"), "Open project", self.load)
 		self.tbSaveAct = self.tb.addAction(getIcon("save"), "Save project", self.save)
 		self.tb.addSeparator()
-		self.tb.addAction(getIcon("stdlib"), "Standard library", self.openLibrary)
+		self.tbUndoAct = self.tb.addAction(getIcon("undo"), "Undo last edit", self.undo)
+		self.tbRedoAct = self.tb.addAction(getIcon("redo"), "Redo", self.redo)
+		self.tb.addSeparator()
+		self.tbCutAct = self.tb.addAction(getIcon("cut"), "Cut", self.cut)
+		self.tbCopyAct = self.tb.addAction(getIcon("copy"), "Copy", self.copy)
+		self.tbPasteAct = self.tb.addAction(getIcon("paste"), "Paste", self.paste)
+		self.tb.addSeparator()
+		self.tbLibAct = self.tb.addAction(getIcon("stdlib"), "Standard library", self.openLibrary)
 		self.addToolBar(self.tb)
 
 		self.__dirtyChanged(False)
+		self.__selResourceChanged(None)
+		self.__textFocusChanged(False)
+		self.__undoAvailableChanged(False)
+		self.__redoAvailableChanged(False)
+		self.__copyAvailableChanged(False)
 
 		self.centralWidget().dirtyChanged.connect(self.__dirtyChanged)
 		self.centralWidget().runStateChanged.connect(self.__runStateChanged)
+		self.centralWidget().textFocusChanged.connect(self.__textFocusChanged)
+		self.centralWidget().selResourceChanged.connect(self.__selResourceChanged)
+		self.centralWidget().undoAvailableChanged.connect(self.__undoAvailableChanged)
+		self.centralWidget().redoAvailableChanged.connect(self.__redoAvailableChanged)
+		self.centralWidget().copyAvailableChanged.connect(self.__copyAvailableChanged)
 
 		if awlSource:
 			self.centralWidget().loadFile(awlSource)
@@ -350,6 +421,65 @@ class MainWindow(QMainWindow):
 				    (VERSION_MAJOR, VERSION_MINOR,
 				     postfix))
 
+	def __updateLibActions(self):
+		# Enable/disable the library toolbar button.
+		# The menu library button is always available on purpose.
+		if self.tbLibAct.isEnabled():
+			if self.__selProjectResource != ProjectWidget.RES_SOURCES:
+				self.tbLibAct.setEnabled(False)
+				self.tbLibAct.setToolTip("Standard library.\n"
+					"Please click in the AWL/STL source "
+					"code at the place where to paste the "
+					"library call.")
+		else:
+			if self.__selProjectResource == ProjectWidget.RES_SOURCES and\
+			   self.__sourceTextHasFocus:
+				self.tbLibAct.setEnabled(True)
+				self.tbLibAct.setToolTip("Standard library")
+
+	def __textFocusChanged(self, textHasFocus):
+		self.__sourceTextHasFocus = textHasFocus
+		self.__updateLibActions()
+
+	def __selResourceChanged(self, resourceNumber):
+		self.__selProjectResource = resourceNumber
+		self.__updateLibActions()
+
+	def __updateUndoActions(self):
+		self.undoAct.setEnabled(
+			self.__undoAvailable and
+			self.__selProjectResource == ProjectWidget.RES_SOURCES
+		)
+		self.tbUndoAct.setEnabled(self.undoAct.isEnabled())
+
+	def __undoAvailableChanged(self, undoAvailable):
+		self.__undoAvailable = undoAvailable
+		self.__updateUndoActions()
+
+	def __updateRedoActions(self):
+		self.redoAct.setEnabled(
+			self.__redoAvailable and
+			self.__selProjectResource == ProjectWidget.RES_SOURCES
+		)
+		self.tbRedoAct.setEnabled(self.redoAct.isEnabled())
+
+	def __redoAvailableChanged(self, redoAvailable):
+		self.__redoAvailable = redoAvailable
+		self.__updateRedoActions()
+
+	def __updateClipboardActions(self):
+		self.cutAct.setEnabled(
+			self.__copyAvailable and
+			self.__selProjectResource == ProjectWidget.RES_SOURCES
+		)
+		self.tbCutAct.setEnabled(self.cutAct.isEnabled())
+		self.copyAct.setEnabled(self.cutAct.isEnabled())
+		self.tbCopyAct.setEnabled(self.copyAct.isEnabled())
+
+	def __copyAvailableChanged(self, copyAvailable):
+		self.__copyAvailable = copyAvailable
+		self.__updateClipboardActions()
+
 	def closeEvent(self, ev):
 		cpuWidget = self.centralWidget().getCpuWidget()
 		if cpuWidget.getState() != CpuWidget.STATE_STOP:
@@ -380,13 +510,16 @@ class MainWindow(QMainWindow):
 		ev.accept()
 		QMainWindow.closeEvent(self, ev)
 
+	def projectHome(self):
+		QDesktopServices.openUrl(QUrl(AWLSIM_HOME_URL, QUrl.StrictMode))
+
 	def about(self):
 		QMessageBox.about(self, "About AWL/STL soft-PLC",
 			"Awlsim soft-PLC version %d.%d\n"
 			"\n"
 			"Copyright 2012-2014 Michael Büsch <m@bues.ch>\n"
 			"\n"
-			"Project home:  http://bues.ch/h/awlsim\n"
+			"Project home:  %s\n"
 			"\n"
 			"\n"
 			"This program is free software; you can redistribute it and/or modify "
@@ -402,7 +535,8 @@ class MainWindow(QMainWindow):
 			"You should have received a copy of the GNU General Public License along "
 			"with this program; if not, write to the Free Software Foundation, Inc., "
 			"51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA." %\
-			(VERSION_MAJOR, VERSION_MINOR))
+			(VERSION_MAJOR, VERSION_MINOR,
+			 AWLSIM_HOME_URL))
 
 	def new(self):
 		self.centralWidget().newFile()
@@ -416,6 +550,9 @@ class MainWindow(QMainWindow):
 	def saveAs(self):
 		self.centralWidget().save(True)
 
+	def guiConfig(self):
+		self.centralWidget().guiConfig()
+
 	def cpuConfig(self):
 		self.centralWidget().cpuConfig()
 
@@ -425,6 +562,21 @@ class MainWindow(QMainWindow):
 	def openLibrary(self):
 		self.centralWidget().openLibrary()
 
+	def undo(self):
+		self.centralWidget().undo()
+
+	def redo(self):
+		self.centralWidget().redo()
+
+	def cut(self):
+		self.centralWidget().cut()
+
+	def copy(self):
+		self.centralWidget().copy()
+
+	def paste(self):
+		self.centralWidget().paste()
+
 # If invoked as script, run a new instance.
 if __name__ == "__main__":
 	fn = sys.argv[1] if (len(sys.argv) >= 2) else None
