@@ -95,6 +95,24 @@ class ProjectWidget(QTabWidget):
 	visibleLinesChanged = Signal(AwlSource, int, int)
 	# Signal: The library selection changed
 	libTableChanged = Signal()
+	# Signal: Source text focus changed
+	textFocusChanged = Signal(bool)
+	# Signal: The selected project resource changed
+	#         Parameter is one of RES_...
+	selResourceChanged = Signal(int)
+	# Signal: UndoAvailable state changed
+	undoAvailableChanged = Signal(bool)
+	# Signal: RedoAvailable state changed
+	redoAvailableChanged = Signal(bool)
+	# Signal: CopyAvailable state changed
+	copyAvailableChanged = Signal(bool)
+
+	# Project resource identifier
+	EnumGen.start
+	RES_SOURCES	= EnumGen.item # Sources
+	RES_SYMTABS	= EnumGen.item # Symbol tables
+	RES_LIBSELS	= EnumGen.item # Library selections
+	EnumGen.end
 
 	def __init__(self, parent=None):
 		QTabWidget.__init__(self, parent)
@@ -112,11 +130,64 @@ class ProjectWidget(QTabWidget):
 
 		self.reset()
 
+		self.currentChanged.connect(self.__handleTabChange)
 		self.awlTabs.sourceChanged.connect(self.codeChanged)
 		self.awlTabs.visibleLinesChanged.connect(self.visibleLinesChanged)
+		self.awlTabs.focusChanged.connect(self.textFocusChanged)
+		self.awlTabs.undoAvailableChanged.connect(self.undoAvailableChanged)
+		self.awlTabs.redoAvailableChanged.connect(self.redoAvailableChanged)
+		self.awlTabs.copyAvailableChanged.connect(self.copyAvailableChanged)
+		self.awlTabs.resizeFont.connect(self.__doSourceCodeFontResize)
 		self.symTabs.sourceChanged.connect(self.symTabChanged)
 		self.libTable.model().contentChanged.connect(self.libTableChanged)
 
+		# Send an initial tab-change notification signal.
+		QTimer.singleShot(0,
+			lambda: self.__handleTabChange(self.currentIndex()))
+
+	# Resize project editor font.
+	def __doSourceCodeFontResize(self, bigger):
+		font = getDefaultFixedFont()
+		fontStr = self.__project.getGuiSettings().getEditorFont()
+		if fontStr:
+			font.fromString(fontStr)
+			font.setStyleHint(QFont.Courier)
+		if bigger:
+			font.setPointSize(font.pointSize() + 1)
+			if font.pointSize() > 72:
+				return
+		else:
+			font.setPointSize(font.pointSize() - 1)
+			if font.pointSize() < 6:
+				return
+		self.__project.getGuiSettings().setEditorFont(font.toString())
+		self.setSettings(self.__project.getGuiSettings())
+
+		self.codeChanged.emit()
+		self.symTabChanged.emit()
+
+	def __handleTabChange(self, newTabIndex):
+		widget = self.widget(newTabIndex)
+		if not widget:
+			return
+		if widget is self.awlTabs:
+			self.selResourceChanged.emit(self.RES_SOURCES)
+			self.undoAvailableChanged.emit(self.awlTabs.undoIsAvailable())
+			self.redoAvailableChanged.emit(self.awlTabs.redoIsAvailable())
+			self.copyAvailableChanged.emit(self.awlTabs.copyIsAvailable())
+		elif widget is self.symTabs:
+			self.selResourceChanged.emit(self.RES_SYMTABS)
+			self.undoAvailableChanged.emit(self.symTabs.undoIsAvailable())
+			self.redoAvailableChanged.emit(self.symTabs.redoIsAvailable())
+			self.copyAvailableChanged.emit(self.symTabs.copyIsAvailable())
+		elif widget is self.libTable:
+			self.selResourceChanged.emit(self.RES_LIBSELS)
+			self.undoAvailableChanged.emit(False)
+			self.redoAvailableChanged.emit(False)
+			self.copyAvailableChanged.emit(False)
+		else:
+			assert(0)
+
 	def handleOnlineDiagChange(self, enabled):
 		self.awlTabs.handleOnlineDiagChange(enabled)
 
@@ -151,8 +222,13 @@ class ProjectWidget(QTabWidget):
 		self.awlTabs.reset()
 		self.symTabs.reset()
 
+	def setSettings(self, guiSettings):
+		self.awlTabs.setSettings(guiSettings)
+		self.symTabs.setSettings(guiSettings)
+
 	def __loadProject(self, project):
 		self.__project = project
+		self.setSettings(project.getGuiSettings())
 		self.awlTabs.setSources(self.__project.getAwlSources())
 		self.symTabs.setSources(self.__project.getSymTabSources())
 		self.libTable.model().setLibSelections(self.__project.getLibSelections())
@@ -362,3 +438,28 @@ class ProjectWidget(QTabWidget):
 				# Add a library selection to the library table.
 				if not self.__pasteLibSel(dlg.pasteLibSel):
 					return
+
+	def undo(self):
+		widget = self.currentWidget()
+		if widget:
+			widget.undo()
+
+	def redo(self):
+		widget = self.currentWidget()
+		if widget:
+			widget.redo()
+
+	def clipboardCut(self):
+		widget = self.currentWidget()
+		if widget:
+			widget.clipboardCut()
+
+	def clipboardCopy(self):
+		widget = self.currentWidget()
+		if widget:
+			widget.clipboardCopy()
+
+	def clipboardPaste(self):
+		widget = self.currentWidget()
+		if widget:
+			widget.clipboardPaste()
