@@ -101,11 +101,24 @@ class SourceTabWidget(QTabWidget):
 
 	# Signal: Emitted, if the source code changed.
 	sourceChanged = Signal()
+	# Signal: Keyboard focus in/out event.
+	focusChanged = Signal(bool)
+	# Signal: UndoAvailable state changed
+	undoAvailableChanged = Signal(bool)
+	# Signal: RedoAvailable state changed
+	redoAvailableChanged = Signal(bool)
+	# Signal: CopyAvailable state changed
+	copyAvailableChanged = Signal(bool)
+	# Signal: Change the font size.
+	#         If the parameter is True, increase font size.
+	resizeFont = Signal(bool)
 
 	def __init__(self, itemName, parent=None):
 		QTabWidget.__init__(self, parent)
 		self.itemName = itemName
 
+		self.guiSettings = GuiSettings()
+
 		self.contextMenu = SourceTabContextMenu(itemName, self)
 
 		self.setMovable(True)
@@ -141,11 +154,6 @@ class SourceTabWidget(QTabWidget):
 		for i in range(self.count()):
 			yield self.widget(i)
 
-	def clear(self):
-		for widget in self.allTabWidgets():
-			widget.deleteLater()
-		QTabWidget.clear(self)
-
 	def updateRunState(self, newRunState):
 		pass
 
@@ -156,6 +164,9 @@ class SourceTabWidget(QTabWidget):
 	def setSources(self, sources):
 		raise NotImplementedError
 
+	def setSettings(self, guiSettings):
+		self.guiSettings = guiSettings
+
 	def integrateSource(self):
 		curWidget = self.currentWidget()
 		if curWidget:
@@ -170,6 +181,30 @@ class SourceTabWidget(QTabWidget):
 			# Tab context menu was requested.
 			self.contextMenu.exec_(self.mapToGlobal(ev.pos()))
 
+	def undoIsAvailable(self):
+		return False
+
+	def undo(self):
+		pass
+
+	def redoIsAvailable(self):
+		return False
+
+	def redo(self):
+		pass
+
+	def copyIsAvailable(self):
+		return False
+
+	def clipboardCut(self):
+		pass
+
+	def clipboardCopy(self):
+		pass
+
+	def clipboardPaste(self):
+		pass
+
 class AwlSourceTabWidget(SourceTabWidget):
 	"AWL source tab-widget"
 
@@ -194,6 +229,11 @@ class AwlSourceTabWidget(SourceTabWidget):
 		self.onlineDiagEnabled = False
 		self.addEditWidget()
 
+	def clear(self):
+		for editWidget in self.allTabWidgets():
+			editWidget.shutdown()
+		SourceTabWidget.clear(self)
+
 	def __emitVisibleLinesSignal(self):
 		editWidget = self.currentWidget()
 		if editWidget:
@@ -212,6 +252,14 @@ class AwlSourceTabWidget(SourceTabWidget):
 				editWidget = self.widget(index)
 				editWidget.enableCpuStats(True)
 		self.__emitVisibleLinesSignal()
+		# Update undo/redo/copy states
+		editWidget = self.currentWidget()
+		self.undoAvailableChanged.emit(editWidget.undoIsAvailable()
+					       if editWidget else False)
+		self.redoAvailableChanged.emit(editWidget.redoIsAvailable()
+					       if editWidget else False)
+		self.copyAvailableChanged.emit(editWidget.copyIsAvailable()
+					       if editWidget else False)
 
 	def updateRunState(self, newRunState):
 		for editWidget in self.allTabWidgets():
@@ -241,10 +289,22 @@ class AwlSourceTabWidget(SourceTabWidget):
 		self.updateActionMenu()
 		self.setCurrentIndex(0)
 
+	def setSettings(self, guiSettings):
+		SourceTabWidget.setSettings(self, guiSettings)
+
+		for editWidget in self.allTabWidgets():
+			editWidget.setSettings(guiSettings)
+
 	def addEditWidget(self):
 		editWidget = EditWidget(self)
+		editWidget.setSettings(self.guiSettings)
 		editWidget.codeChanged.connect(self.sourceChanged)
+		editWidget.focusChanged.connect(self.focusChanged)
 		editWidget.visibleRangeChanged.connect(self.__emitVisibleLinesSignal)
+		editWidget.undoAvailable.connect(self.undoAvailableChanged)
+		editWidget.redoAvailable.connect(self.redoAvailableChanged)
+		editWidget.copyAvailable.connect(self.copyAvailableChanged)
+		editWidget.resizeFont.connect(self.resizeFont)
 		index = self.addTab(editWidget, editWidget.getSource().name)
 		self.setCurrentIndex(index)
 		self.updateActionMenu()
@@ -260,6 +320,7 @@ class AwlSourceTabWidget(SourceTabWidget):
 				"Delete source '%s'?" % text,
 				QMessageBox.Yes, QMessageBox.No)
 			if res == QMessageBox.Yes:
+				self.widget(index).shutdown()
 				self.removeTab(index)
 				self.sourceChanged.emit()
 
@@ -316,7 +377,50 @@ class AwlSourceTabWidget(SourceTabWidget):
 	def pasteText(self, text):
 		editWidget = self.currentWidget()
 		if editWidget:
-			editWidget.insertPlainText(text)
+			editWidget.pasteText(text)
+
+	def undoIsAvailable(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			return editWidget.undoIsAvailable()
+		return False
+
+	def undo(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			editWidget.undo()
+
+	def redo(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			editWidget.redo()
+
+	def redoIsAvailable(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			return editWidget.redoIsAvailable()
+		return False
+
+	def copyIsAvailable(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			return editWidget.copyIsAvailable()
+		return False
+
+	def clipboardCut(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			editWidget.cut()
+
+	def clipboardCopy(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			editWidget.copy()
+
+	def clipboardPaste(self):
+		editWidget = self.currentWidget()
+		if editWidget:
+			editWidget.paste()
 
 class SymSourceTabWidget(SourceTabWidget):
 	"Symbol table source tab-widget"	
@@ -352,6 +456,7 @@ class SymSourceTabWidget(SourceTabWidget):
 		symTabView = SymTabView(self)
 		symTabView.setSymTab(SymbolTable())
 		symTabView.model().sourceChanged.connect(self.sourceChanged)
+		symTabView.focusChanged.connect(self.focusChanged)
 		index = self.addTab(symTabView, symTabView.model().getSource().name)
 		self.setCurrentIndex(index)
 		self.updateActionMenu()
